import Link from "next/link";
import Image from "next/image";

import { User } from "@privy-io/react-auth";

import { motion } from "framer-motion";

import { toast } from "sonner";

import { Pencil, Play, Share2 } from "lucide-react";

import { Tables } from "@/utils/supabase/database.types";

import { cn, formatDate } from "@/lib/utils";
import { ANIMATION_EASE } from "@/lib/config";

import { Button, buttonVariants } from "@/components/ui/button";

type AgentCardProps = {
  data: Tables<"agents">;
  idx: number;
  user: User | null;
};

export default function AgentCard(props: AgentCardProps) {
  const {
    id,
    image_url,
    name,
    description,
    model_type,
    created_at,
    conversation,
    last_conv,
    user_id,
  } = props.data;

  return (
    <motion.li
      initial={{ opacity: 0, y: 20 }}
      animate={{ opacity: 1, y: 0 }}
      transition={{
        duration: 0.8,
        delay: props.idx * 0.2,
        ease: ANIMATION_EASE,
      }}
      className={cn(
        "flex flex-col p-4",
        "bg-background rounded-lg transition-all duration-300 border hover:shadow-[0_0_23.1px_0_rgba(0,0,0,0.08)_inset]"
      )}
    >
      {image_url && (
        <Image
          className="object-cover mb-3 rounded-lg overflow-hidden"
          src={"/agent.png"}
          width={1200}
          height={630}
          alt={name}
        />
      )}
      <div className="flex items-center justify-between">
        <h3 className="text-xl font-bold mb-2">{name}</h3>
        <div
          className="mb-2 text-white px-2 py-0.5 rounded-md"
          style={{
            background: "linear-gradient(90deg, #FF512F 0%, #F09819 100%)",
          }}
        >
          {conversation} chats
        </div>
      </div>
      <p className="text-sm text-muted-foreground mb-2">{description}</p>
      <p className="mb-2 text-primary">
        <time dateTime={created_at} className="text-xs">
          <span className="font-bold">Last Active: </span>
          {last_conv ? formatDate(last_conv) : "-"}
        </time>
      </p>
      <div className="mt-auto grid sm:grid-cols-2 gap-2">
        <Link
          href={`/agents/${id}`}
          target="blank"
          className={buttonVariants({ variant: "default" })}
        >
          <svg
            width="16"
            height="16"
            viewBox="0 0 17 20"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
            className="mr-2"
          >
            <path
              d="M16.75 9.99998C16.7506 10.2546 16.6853 10.5051 16.5605 10.727C16.4356 10.949 16.2555 11.1348 16.0375 11.2665L2.53 19.5297C2.30227 19.6691 2.04144 19.7452 1.77445 19.7502C1.50746 19.7551 1.24399 19.6887 1.01125 19.5578C0.780728 19.4289 0.588696 19.2409 0.454904 19.0132C0.321111 18.7855 0.250385 18.5263 0.25 18.2622V1.73779C0.250385 1.47368 0.321111 1.21445 0.454904 0.986734C0.588696 0.759022 0.780728 0.571057 1.01125 0.442167C1.24399 0.311244 1.50746 0.244825 1.77445 0.249769C2.04144 0.254713 2.30227 0.330842 2.53 0.470292L16.0375 8.73342C16.2555 8.86511 16.4356 9.05097 16.5605 9.27291C16.6853 9.49485 16.7506 9.74533 16.75 9.99998Z"
              fill="white"
            />
          </svg>

          <span>Try it</span>
        </Link>
        <Button
          variant={"outline"}
          onClick={() => {
            navigator.clipboard.writeText(
              `${process.env.NEXT_PUBLIC_SITE_URL}/agents/${id}`
            );
            toast.info("Copied to clipboard!");
          }}
        >
          <svg
            width="16"
            height="16"
            viewBox="0 0 19 22"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
            className="mr-2"
          >
            <path
              d="M18.125 17.7502C18.1251 18.2428 18.0173 18.7294 17.8093 19.1759C17.6012 19.6225 17.298 20.018 16.9208 20.3349C16.5436 20.6517 16.1016 20.8821 15.6259 21.01C15.1502 21.1378 14.6522 21.16 14.167 21.0749C13.6818 20.9898 13.2211 20.7995 12.8173 20.5174C12.4135 20.2353 12.0763 19.8682 11.8294 19.442C11.5825 19.0157 11.432 18.5405 11.3883 18.0499C11.3446 17.5592 11.4089 17.0649 11.5766 16.6017L6.60782 13.4095C6.13327 13.875 5.53182 14.19 4.87893 14.315C4.22604 14.44 3.55079 14.3693 2.93788 14.112C2.32497 13.8547 1.8017 13.422 1.43373 12.8684C1.06575 12.3148 0.869446 11.6649 0.869446 11.0002C0.869446 10.3354 1.06575 9.68549 1.43373 9.13188C1.8017 8.57828 2.32497 8.14568 2.93788 7.88833C3.55079 7.63099 4.22604 7.56038 4.87893 7.68534C5.53182 7.81031 6.13327 8.1253 6.60782 8.59079L11.5766 5.40329C11.292 4.62146 11.3056 3.7622 11.6145 2.98971C11.9235 2.21722 12.5063 1.58565 13.2515 1.2157C13.9967 0.845739 14.8521 0.76333 15.6543 0.984215C16.4564 1.2051 17.149 1.71379 17.5998 2.41309C18.0506 3.11239 18.2279 3.95326 18.0979 4.77503C17.9678 5.5968 17.5396 6.34185 16.8949 6.86781C16.2503 7.39378 15.4344 7.66378 14.6033 7.62622C13.7721 7.58867 12.984 7.24619 12.3894 6.66423L7.42063 9.85642C7.68917 10.5985 7.68917 11.4112 7.42063 12.1533L12.3894 15.3455C12.8638 14.8812 13.4646 14.5671 14.1165 14.4425C14.7685 14.3179 15.4428 14.3884 16.055 14.645C16.6671 14.9017 17.19 15.3331 17.5582 15.8855C17.9264 16.4378 18.1236 17.0864 18.125 17.7502Z"
              fill="#222222"
            />
          </svg>
          <span>Share</span>
        </Button>
      </div>
      {!props.user || user_id !== props.user.id ? (
        <Button className="mt-2" variant={"dots"} disabled={true}>
          <svg
            width="16"
            height="16"
            viewBox="0 0 19 19"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
            className="mr-2"
          >
            <path
              d="M18.3103 4.87866L14.1216 0.688973C13.9823 0.549649 13.8169 0.439129 13.6349 0.363725C13.4529 0.288322 13.2578 0.249512 13.0608 0.249512C12.8638 0.249512 12.6687 0.288322 12.4867 0.363725C12.3047 0.439129 12.1393 0.549649 12 0.688973L0.439695 12.2502C0.299801 12.389 0.188889 12.5542 0.113407 12.7362C0.0379245 12.9183 -0.000621974 13.1135 7.58902e-06 13.3105V17.5002C7.58902e-06 17.898 0.158043 18.2796 0.439347 18.5609C0.720652 18.8422 1.10218 19.0002 1.50001 19.0002H5.6897C5.88675 19.0009 6.08197 18.9623 6.26399 18.8868C6.44602 18.8113 6.61122 18.7004 6.75001 18.5605L18.3103 7.00022C18.4496 6.86093 18.5602 6.69556 18.6356 6.51355C18.711 6.33153 18.7498 6.13645 18.7498 5.93944C18.7498 5.74243 18.711 5.54735 18.6356 5.36534C18.5602 5.18333 18.4496 5.01795 18.3103 4.87866ZM15 8.18897L10.8103 4.00022L13.0603 1.75022L17.25 5.93897L15 8.18897Z"
              fill="#222222"
            />
          </svg>
          Edit
        </Button>
      ) : (
        <Link
          href={`/agents/${id}/edit`}
          className={cn("mt-2", buttonVariants({ variant: "dots" }))}
        >
          <svg
            width="16"
            height="16"
            viewBox="0 0 19 19"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
            className="mr-2"
          >
            <path
              d="M18.3103 4.87866L14.1216 0.688973C13.9823 0.549649 13.8169 0.439129 13.6349 0.363725C13.4529 0.288322 13.2578 0.249512 13.0608 0.249512C12.8638 0.249512 12.6687 0.288322 12.4867 0.363725C12.3047 0.439129 12.1393 0.549649 12 0.688973L0.439695 12.2502C0.299801 12.389 0.188889 12.5542 0.113407 12.7362C0.0379245 12.9183 -0.000621974 13.1135 7.58902e-06 13.3105V17.5002C7.58902e-06 17.898 0.158043 18.2796 0.439347 18.5609C0.720652 18.8422 1.10218 19.0002 1.50001 19.0002H5.6897C5.88675 19.0009 6.08197 18.9623 6.26399 18.8868C6.44602 18.8113 6.61122 18.7004 6.75001 18.5605L18.3103 7.00022C18.4496 6.86093 18.5602 6.69556 18.6356 6.51355C18.711 6.33153 18.7498 6.13645 18.7498 5.93944C18.7498 5.74243 18.711 5.54735 18.6356 5.36534C18.5602 5.18333 18.4496 5.01795 18.3103 4.87866ZM15 8.18897L10.8103 4.00022L13.0603 1.75022L17.25 5.93897L15 8.18897Z"
              fill="#222222"
            />
          </svg>
          Edit
        </Link>
      )}
    </motion.li>
  );
}
